//===========================================================================
//                                                                       GRID
//                                                                14 May 2003
//                                                               Nick Gessler
//                                      Inspired by Casey Reas' "Distance 2D"
//                   http://proce55ing.net/learning/examples/distance_2d.html
//
//      IMPORTANT:
//         In order to pass the myShapesMouseMove to FormMouseMove,
//         Three blocks of code need to be inserted by hand:
//
//         1)           Using NOTEPAD to edit Unit1.h, insert after "public:"
//                          void __fastcall myShapesMouseMov(TObject *Sender,
//                                          TSHiftState Shift, int X, int Y);
//         2)            When myShapes are created, insert this line of code:
//                     myShapes[10 * e + s]->OnMouseMove = myShapesMouseMove;
//         3)            Write out a myShapesMouseMove event handler by hand.
//                                                See the code block below...
//
//      TRY:
//                  1) Changing the calls to the color ramp function
//                  2) Making the Top & Left distance factor negative
//                  3) Try doubling the Top & Left distance factors
//                  4) Try dynamically changing the color functions
//                  5) Try dynamically changing any/everything
//===========================================================================

#include <vcl.h>
#pragma hdrstop
#include "Unit1.h"
#include "math.h"                                        // don't forget this
#include "time.h"                                        // don't forget this
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;

//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
   : TForm(Owner)
{
   DoubleBuffered = true;                                // don't forget this
}
//---------------------------------------------------------------------------

//===========================================================================
//                                                           GLOBAL VARIABLES
//===========================================================================

TShape *myShapes[100];
int size;
int distance, twiceDistance, factor;
int x, y;
int gray;
int timerFactor = 20;

//===========================================================================
//                                                                  FUNCTIONS
//===========================================================================

//----------------------------------------------------------- Replot Function
void replot (void) {
   for (int e = 0; e < 10; e++) {
      for (int s = 0; s < 10; s++) {
         distance =  sqrt (pow((x - e), 2) + pow((y - s), 2));
         twiceDistance = distance * 2;
         factor = 25 - 4 * distance;

         myShapes[10 * e + s]->Height = 50 - twiceDistance;
         myShapes[10 * e + s]->Width = 50 - twiceDistance;
         myShapes[10 * e + s]->Top = 50 * s + distance;      // try -
         myShapes[10 * e + s]->Left = 50 * e + distance;     // try -

         myShapes[10 * e + s]->Pen->Width = factor;
         gray = distance * timerFactor;
         myShapes[10 * e + s]->Pen->Color =
            static_cast<TColor>(RGB(gray, 255-gray, gray));
         myShapes[10 * e + s]->Brush->Color =
            static_cast<TColor>(RGB(gray, 255-gray, 255-gray));
      }
   }
}

//===========================================================================
//                                                             EVENT HANDLERS
//===========================================================================

//-------------------------------------------------------------- On Form Show
void __fastcall TForm1::FormShow(TObject *Sender)
{
      for (int e = 0; e < 10; e++) {
         for (int s = 0; s < 10; s++) {
            myShapes[10 * e + s] = new TShape(this);
            myShapes[10 * e + s]->Parent = Form1;
            myShapes[10 * e + s]->Visible = true;
            myShapes[10 * e + s]->Top = 50 * s;
            myShapes[10 * e + s]->Left = 50 * e;
            myShapes[10 * e + s]->Width = 50;
            myShapes[10 * e + s]->Height = 50;
            myShapes[10 * e + s]->Shape = stSquare;
            myShapes[10 * e + s]->Brush->Color = clBlack;
            myShapes[10 * e + s]->Pen->Color = clBlack;
            myShapes[10 * e + s]->Pen->Width = 1;
            myShapes[10 * e + s]->OnMouseMove = myShapesMouseMove;
         }
      }
      randomize();
}

//-----------------------------------------------------On myShapes Mouse Move
//                                You must write this event handler yourself.
//                            It passes myShapesMouseMove on to FormMouseMove
void __fastcall TForm1::myShapesMouseMove(TObject *Sender, TShiftState Shift,
   int X, int Y)
{
   X += ((TShape *) Sender) -> Left;
   Y += ((TShape *) Sender) -> Top;
   FormMouseMove(Sender, Shift, X, Y);
}

//-------------------------------------------------------- On Form Mouse Move
void __fastcall TForm1::FormMouseMove(TObject *Sender, TShiftState Shift,
      int X, int Y)
{
   x = X / 50;
   y = Y / 50;
   replot();
}

//------------------------------------------------------------------ On Timer
//                             Every 3 seconds the coloring algorithm changes
void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
   timerFactor = 20 + random (30);
}

//------------------------------------------------------------- On Form Close
void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
   Application->ProcessMessages();
   for (int i = 0; i < 100; i++) {
      delete myShapes[i];
   }
}

//===========================================================================
//                                                           THAT'S ALL FOLKS
//===========================================================================
